//-----------------------------------------------------------------------------
// Wrapper class for using Windows Services
// 
// Copyright (c) 2001/2002 by Dean Harding <dean@codeka.com>
//-----------------------------------------------------------------------------
#pragma once

//-----------------------------------------------------------------------------
/**
 * This encapsulates everything you need to be able to run as a service.  To
 * use it, simply call the CService::Execute method with the command-line.
 * This will call one of CService::Install, ArmsService::Uninstall,
 * ArmsService::Start, ArmsService::Stop or ArmsService::Run.
 */
class CService
{
protected:
	/**
	 * This is what we want the service to do.  Depending on the value of this,
	 * we may start, stop, install, uninstall or run the service.
	 */
	enum Action
	{
		DoInstall,		///< Install the service
		DoUninstall,	///< Uninstall the service
		DoStart,		///< Start the service
		DoStop,			///< Stop the service
		DoRun,			///< Run the service as a service
		DoConsole,		///< Run the service as a console app (i.e. not as an actual service).
						///< Good for debugging purposes.

		Unknown			///< We don't know what to do!
	};

	/**
	 * This is called by the SCM to start the service.
	 */
	static void WINAPI ServiceDispatch( DWORD numArgs, char **args );

	/**
	 * This is called by the SCM when it want to change the status of the service
	 * (e.g. to stop it, or pause it, etc).
	 */
	static void WINAPI ServiceCtrlHandler( DWORD control );

	/**
	 * Run the service.  This is where all the action happens.  Think of it as
	 * the "entry point" for the service.
	 *
	 * @param numArgs number of arguments to pass to the service
	 * @param args is the arguments for this run
	 * @returns true if the service was started and is running properly, false otherwise.
	 */
	bool Run( int numArgs, std::string *args );

	/**
	 * Install the service.  This will install the service in the SCM applet, so
	 * the user can start and stop it from there.
	 *
	 * @param user is the username to run the service under
	 * @param password is the password to run the service under
	 */
	void Install( std::string const &user, std::string const &password );

	/**
	 * Uninstall the service.  This will remove the service from the SCM applet.
	 */
	void Uninstall();

	/**
	 * Start the service.  This will start the service, just like if the user had
	 * pressed the "Start" button in the SCM applet.
	 */
	void Start();

	/**
	 * Stop the service.  This will stop the service, just like if the user had
	 * pressed the "Stop" button in the SCM applet.
	 */
	void Stop();

	/**
	 * Runs the service as a normal console app.
	 *
	 * @param numArgs number of command-line arguments
	 * @param args command-line arguments
	 */
	void Console( int numArgs, std::string *args );

	/**
	 * If a wrong argument or invalid arguments are passed, this will tell the user
	 * of their mistake.
	 */
	void ShowUsage();

	/**
	 * If there's an error, this will display it.  We don't just write to the log,
	 * since we may also want to pop up a message box as well.
	 */
	static void DisplayError( std::string const &message );

	/**
	 * Shutdown the threads and so on that have been started.
	 */
	void Shutdown();

	/**
	 * Name this service is running as.  This is used so that we can have multiple
	 * services on the same machine, each getting configuration data and so on
	 * from difference sources.
	 */
	std::string m_name;

	/**
	 * The status of the service, as we understand it to be.  This is used by the
	 * ServiceDispatch method to help in controlling the service.
	 */
	SERVICE_STATUS m_status;

	/**
	 * This is the handle to the service's status
	 */
	SERVICE_STATUS_HANDLE m_statusHandle;

	/**
	 * This is used when we're running in "-console" mode, to signal that we should
	 * quit.
	 */
	HANDLE m_quitEvent;

	/**
	 * This is a singleton class.  We use that pattern for accessing it, so this
	 * is the handle to the global ArmsService object.
	 */
	static CService m_handle;

	/**
	 * Constructor is private, since we don't want to be able to contruct an
	 * Arms service object, except in the static variable above.
	 */
	CService();
public:
	~CService();

	/**
	 * Initialize the service then, depending on the command-line, this will also
	 * run the actual service.  Once this function returns, the caller should
	 * exit.
	 *
	 * @param numArgs this is the number of arguments.
	 * @param args this is an array of string, each of which is an argument.
	 */
	void Execute( int numArgs, char **args );

	/**
	 * This will stop and exit the service
	 */
	void ExitService();

	std::string GetName() { return m_name; }
	static CService *GetHandle() { return &m_handle; }
};